package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.access.FixAnalysisAccess;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import de.intevation.flys.utils.KMIndex;

import org.apache.log4j.Logger;

/**
 * Facet to show the outliers in a fix calculation.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixOutlierFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixOutlierFacet.class);

    /** Trivial Constructor. */
    public FixOutlierFacet() {
    }


    /**
     * @param name
     */
    public FixOutlierFacet(String name, String description) {
        super(0, name, description, ComputeType.ADVANCE, null, null);
    }

    public FixOutlierFacet(int index, String name, String description) {
        super(index, name, description, ComputeType.ADVANCE, null, null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixOutlierFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixAnalysisAccess access = new FixAnalysisAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();
            double currentKm =
                ((Double)context.getContextValue("currentKm")).doubleValue();

            KMIndex<QW []> kmQWs = result.getOutliers();

            KMIndex.Entry<QW []> qwsEntry = kmQWs.binarySearch(currentKm);
            QW[] qws = null;
            if(qwsEntry != null) {
                qws = qwsEntry.getValue();
            }
            return qws;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixOutlierFacet deepCopy() {
        FixOutlierFacet copy = new FixOutlierFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
